from active_bo_msgs.srv import WeightToPolicy

import rclpy
from rclpy.node import Node

from active_bo_ros.PolicyModel.GaussianRBFModel import GaussianRBF
import numpy as np

class PolicyService(Node):
    def __init__(self):
        super().__init__('policy_service')
        self.srv = self.create_service(WeightToPolicy, 'policy_srv', self.policy_callback)

    @staticmethod
    def policy_callback(request, response):
        weights = request.weights
        weight_len = len(weights)
        nr_steps = request.nr_steps

        policy = GaussianRBF(weight_len, nr_steps)
        policy.weights = weights
        policy.rollout()

        response.policy = policy.policy.flatten().tolist()
        return response

def main(args=None):
    rclpy.init(args=args)

    policy_service = PolicyService()

    rclpy.spin(policy_service)

    rclpy.shutdown()

if __name__ == "__main__":
    main()