from active_bo_msgs.srv import RLRollOut
from active_bo_msgs.msg import ImageFeedback

import rclpy
from rclpy.node import Node

from active_bo_ros.ReinforcementLearning.ContinuousMountainCar import Continuous_MountainCarEnv

import numpy as np
import time

class RLService(Node):
    def __init__(self):
        super().__init__('rl_service')
        self.srv = self.create_service(RLRollOut, 'rl_srv', self.rl_callback)

        self.publisher = self.create_publisher(ImageFeedback, 'rl_feedback', 1)

        self.env = Continuous_MountainCarEnv(render_mode='rgb_array')
        self.distance_penalty = 0

    def rl_callback(self, request, response):

        feedback_msg = ImageFeedback()

        reward = 0
        step_count = 0
        policy = request.policy

        for i in range(len(policy)):
            action = policy[i]
            output = self.env.step(action)

            reward += output[1]
            done = output[2]
            step_count += 1

            rgb_array = output[5]
            rgb_shape = rgb_array.shape

            feedback_msg.height = rgb_shape[0]
            feedback_msg.width = rgb_shape[1]

            self.publisher.publish(feedback_msg)

            if done:
                break

            distance = -(self.env.goal_position - output[0][0])
            reward += distance * self.distance_penalty

            time.sleep(0.01)

        response.reward = reward
        response.final_step = step_count

        return response

def main(args=None):
    rclpy.init(args=args)

    rl_service = RLService()

    rclpy.spin(rl_service)

if __name__ == '__main__':
    main()
